package RPM::Source::Convert::Plugins::ALT2ALT;
use strict;
use warnings;
use Carp;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory::DependencyFilter;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $cleanup_req_devel=1;
our $cleanup_req_pymod=0;

our @LONGOPT=(
    'cleanup-requires-devel!' => \$cleanup_req_devel,
    'cleanup-requires-python-modules!' => \$cleanup_req_pymod,
    );

sub longopt_usage {
    print "ALT specific cleanup options:\n";
    print "	--cleanup-requires-python-modules. Cleanup Req: from python-module-*.\n";
    print "	--no-cleanup-requires-devel. Do not cleanup Req: from -devel.\n";
}

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'post/preun explicit script dependencies',
    -description => 'drop cleaned up post/preun explicit script dependencies',
    -class => 'ALT2ALT::DependencyFilter::BuildRequiresCleanup',
    -tags => SPEC_TAG_BUILD_REQUIRES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'ALT2ALT::NoDevelReqs',
    -description => 'convert generic Fedora name conventions to alt name conventions',
    -class => 'RPM::Source::Tools::Dependency::Filter::ALT2ALT::NoDevelReqs',
    -tags => SPEC_TAG_REQUIRES,
    -priority => FILTER_PRIORITY_DEFAULT,
);
&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'ALT2ALT::NoPythonModuleReqs',
    -description => 'convert generic Fedora name conventions to alt name conventions',
    -class => 'RPM::Source::Tools::Dependency::Filter::ALT2ALT::NoPythonModuleReqs',
    -tags => SPEC_TAG_REQUIRES,
    -priority => FILTER_PRIORITY_DEFAULT,
);

package RPM::Source::Tools::Dependency::Filter::ALT2ALT::NoDevelReqs;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    return $dep unless $RPM::Source::Convert::Plugins::ALT2ALT::cleanup_req_devel;
    my $pkgname=$dep->{-name};
    if ($pkgname=~/-devel$/) {
	delete $dep->{-name};
    }
    return $dep;
}


package RPM::Source::Tools::Dependency::Filter::ALT2ALT::NoPythonModuleReqs;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    return $dep unless $RPM::Source::Convert::Plugins::ALT2ALT::cleanup_req_pymod;
    my $pkgname=$dep->{-name};
    if ($pkgname=~/^python3?-module-/) {
	delete $dep->{-name};
    }
    return $dep;
}

package ALT2ALT::DependencyFilter::BuildRequiresCleanup;

use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;
our $verbose=0;

## TODO python ??
##
sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    print STDERR "DEBUG: BuildRequiresCleanup: testing [$pkgname]\n" if $verbose>1;
    return if grep {$pkgname eq $_} qw!
make
%{__make}
pkgconfig
!;
    return $dep;
}


1;
