package RPM::Source::Convert::Plugins::DistroMap::SourceAnalyzer;
use strict;
use warnings;
use Carp;

use File::Basename;
use RPM::Source::Tools::SourceBundle;
use RPM::Source::Dependency::Analyzer 0.007;
use RPM::Source::Shared::Resource::DistroMap;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Factory::Buildreq;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

$DistroMap::opt_dest_check_distrodb=7; # days

our $verbose=0;

sub get_longopt {
    return RPM::Source::Dependency::Analyzer->get_longopt;
}
sub before_process_options {
    RPM::Source::Dependency::Analyzer->before_process_options;
}
sub process_options {
    RPM::Source::Dependency::Analyzer->process_options;
}

my ($collector,$resolver);

sub init_factory {
    my (undef,$resourceobj)=@_;
}

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'distromap_source_analyzer',
	    -opt_handler_sub=>\&opt_handler_distromap_source_analyzer),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_DEFAULT,
	    -name => 'distromap_source_analyzer_resolve_umbiguous_pkgconfig',
	    -opt_handler_sub=>\&opt_handler_distromap_source_analyzer_resolve_umbiguous_pkgconfig)
	;
}

sub opt_handler_distromap_source_analyzer {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();

    unless (defined $spec->{SOURCEDIR}) {
	print STDERR "WARNING: SourceAnalyzer: sourcedir is not defined. Can't find the sources.\n";
	return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
    }
    my $name=$spec->main_section->get_tag('Name');
    unless (defined $name) {
	print STDERR "WARN: get_tag('Name') failed: something wrong with the spec?\n" if $RPM::Source::Transform::verbose;
    } elsif ($name =~ /^mingw/) {
	print STDERR "info: SourceAnalyzer: mingw support is not implemented.\n" if $RPM::Source::Transform::verbose;
	return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
    }
    
    my $storage=RPM::Source::Dependency::Analyzer::Storage->new();
    $collector||=RPM::Source::Dependency::Analyzer::Collector->new(
	-verbose => $RPM::Source::Transform::verbose,
	# useless hint: better be hint guess from buildreq-src
	-hint => 'convert',
	-scan_configure_for_broken_cpp_check => 1,
	);
    foreach my $srcitem (@{$spec->raw_sourcelist_ref}) {
	next unless $srcitem;
	print STDERR "DistroMap::SourceAnalyzer: looking for $srcitem\n" if $verbose;
	$srcitem=$macros->macro_subst($srcitem);
	# due to something like %{download_root}MTLc3m.ttf
	my $filename=$spec->{SOURCEDIR}.'/'.basename($srcitem);
	print STDERR "DistroMap::SourceAnalyzer: no file $filename:\n" if not -f $filename and ($verbose or $RPM::Source::Transform::verbose);
	next unless -e $filename;
	my $tarball=RPM::Source::Tools::SourceBundle->create($filename);
	print STDERR "DistroMap::SourceAnalyzer: analyzing $filename\n" if $verbose;
	$collector->collect_source_dependencies($storage,$tarball);
    }

    ###########################################################################################
    ###########################################################################################
    # TODO: somehow incorporate $existing,$existing_pre/ and a future code from buildreq-src!!!
    #my ($existing,$existing_pre)=$resourceobj->buildreq->current_buildreq($spec);
    ###########################################################################################
    ###########################################################################################
    $resolver||=RPM::Source::Dependency::Analyzer::Resolver->new(
	-verbose => $RPM::Source::Transform::verbose,
	-distroinfo=> $resourceobj->get_dest_distroinfo(),
	);
    $resolver->resolve_storage($storage,$resourceobj->buildreq);

    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

# TODO! Disrtomap it! or move to SourceAnalyzer internal hack option!!
my %dep_subst_hack=qw/
pkgconfig(pygobject-2.0) python-module-pygobject-devel
perl(mro.pm) perl-base
/;

#AC_CHECK_PROGS(xterm, Eterm gnome-terminal aterm rxvt xterm)

my %dep_drop_hack=map{$_=>1} qw!
pkgconfig(hal)
/usr/bin/xemacs
/usr/bin/Eterm
!;

sub opt_handler_distromap_source_analyzer_resolve_umbiguous_pkgconfig {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $buildreq = $resourceobj->buildreq;
    foreach my $dep ($buildreq->get_list_buildreq) {
	if ($dep_subst_hack{$dep}) {
	    $buildreq->del_buildreq($dep);
	    $buildreq->add_buildreq_for("replaced automatic dependency $dep", $dep_subst_hack{$dep});
	    print STDERR "info: resolved dependency $dep into $dep_subst_hack{$dep}.\n" if $RPM::Source::Transform::verbose;
	} elsif ($dep_drop_hack{$dep}) {
	    $buildreq->del_buildreq($dep);
	    print STDERR "info: dropped dependency $dep\n" if $RPM::Source::Transform::verbose;
	}
    }
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}


1;
