package RPM::Source::Convert::Plugins::Mageia2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Shared::Resource::DistroMap;
use RPM::Source::Convert::Plugins::Generic2ALT;
# after Generic2ALT; not to disturb the order
use RPM::Source::Convert::Plugins::PerlAutodepsUseStyle2PathStyle;
use RPM::Source::Transformation::Factory::DependencyFilter;


our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Mageia Perl versions',
    -description => 'convert Mageia specific perl versions',
    -class => 'Mageia2ALT::DependencyFilter::PerlVer',
    -tags => SPEC_TAG_BUILD_REQUIRES | SPEC_TAG_REQUIRES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);


sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'mageia2alt_preconvert',
	    -opt_handler_sub=>\&opt_handler_mageia2alt_preconvert),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'mageia2alt_convert',
	    -opt_handler_sub=>\&opt_handler_mageia2alt_convert);
}

# mklibname should be handled before DistroMapping deps
sub opt_handler_mageia2alt_preconvert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    $sec->map_body(sub {
		# mandrake mklibname macros
		#s/^(\s*\%define\s+\S+\s+)\%mklibname\s+(\S+)(?:\s+(\S+))?(.*)$/$1&__mk/
		s/\%mklibname\s+(\S+)(?:\s+(\S+))?/&__mklibname($1,$2)/ge;
			   });
	}
    }
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

sub opt_handler_mageia2alt_convert {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;

    my $macros=$spec->macros();
    my $mainsec=$spec->main_section;

    my $has_perl_convert_version=0;
    my ($flag_need_mageia_compat, $flag_need_cmake);

    my @__push_mainsec;
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    $sec->map_body(sub {
		s/\%mkrel\s+//g;
		# our perl policy is different
		$has_perl_convert_version=1 if s/\%perl_convert_version\s+//g;

#		if (/^BuildRequires:/i) {
#		} elsif (/^Requires:/i) {
#		}
			   } );
	} elsif ($t eq 'prep') {
	    $sec->map_body(sub {
		if (s/(?<!\%)\%apply_patches/#%%apply_patches/) {
		    my $patchlist=$spec->raw_patchlist_ref;
		    for (my $i=0; $i <@$patchlist;$i++) {
			next unless $patchlist->[$i];
			$_.='%patch'.$i.' -p1'."\n";
		    }
		}
		if (s/(?<!%)\%autosetup\b/%setup/) {
		    my $patcharg='';
		    $patcharg=$1 if s/\s(-p\d+)\b/ /;
		    my $patchlist=$spec->raw_patchlist_ref;
		    for (my $i=0; $i <@$patchlist;$i++) {
			next unless $patchlist->[$i];
			$_.='%patch'.$i.' '.$patcharg."\n";
		    }
		}


			   });
	} elsif ($t eq 'build') {
	} elsif ($t eq 'install') {
	    $sec->map_body(sub {
		s,\%make_install(?=(?:\s|$)),%make_install DESTDIR=%buildroot install,g
			   });
	} elsif ($t eq 'triggerun') {
	    if ($sec->match_body(qr'(?:chkconfig|systemctl)')) {
		$sec->delete;
		next;
	    }
	} elsif ($t eq 'post') {
	    $sec->map_body(sub {
		s,\%_post_service,%post_service,g;
			   } );
	} elsif ($t eq 'preun') {
	    $sec->map_body(sub {
		s,\%_preun_service,%preun_service,g;
			   } );
	} elsif ($t eq 'files') {
	    #
#%{_libdir}/*.so.%{major}{,.*}
	    $sec->map_body(sub {
		if (s!(\%\{_libdir\}/\*\.so\.\%\{major\}\{,\.\*\})!%{_libdir}/*.so.*!g) {
		    warn "INFO: $1 not supported in rpm 4.0.4 -- reduced.\n";
		}
		# hack; found in perl-B-C(-devel)
		s,%\{perl_root\}/%\{perl_version\}/%\{full_arch\},%{perl_vendor_archlib},;
			   } );

	}

	$sec->map_body(sub {
	    # clean up
	    s,\%{\??perl_default_filter},,g;

	    s,\%configure2_5x,%configure,g;
	    s,\%compat_makeinstall_std,%makeinstall_std,g;
	    s,\%{compat_makeinstall_std},%{makeinstall_std},g;
	    if (s/(?<!%)\%\{?cmake\}?(?=\s|$)/%{mageia_cmake}/g) {
		$flag_need_mageia_compat=1;
		$flag_need_cmake=1;
	    }
	    $flag_need_cmake=1 if s,\%{__cmake},cmake,g;
	} ) if ($t eq 'install' or $t eq 'build' or $t eq 'prep');

    }
    $mainsec->push_body(@__push_mainsec) if @__push_mainsec;
    if ($has_perl_convert_version) {
	my $macros=$spec->macros();
	my $ver=$macros->get_raw('version');
	$ver=~s/^(?:\%perl_convert_version)?\s+//;
	$macros->set_raw('version',$ver);
    }

    my $buildreq=$resourceobj->buildreq;
    $buildreq->add_pre_buildreq('rpm-macros-mageia-compat') if $flag_need_mageia_compat;
    $buildreq->add_buildreq('cmake') if $flag_need_cmake; # like kde apps ...

    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

sub __mklibname {
    my ($arg1,$arg2)=@_;
    $arg1=&__brace_macro($arg1);
    $arg2=&__brace_macro($arg2);
    my $out;
    if ($arg1 eq '-d') {
	$out='lib'.$arg2.'-devel';
    } elsif ($arg2 eq '-d') {
	$out='lib'.$arg1.'-devel';
    } elsif (defined $arg2) {
	$out='lib'.$arg1.$arg2;
    } else {
	$out='lib'.$arg1;
    }
    return $out;
}


sub __brace_macro {
    my $in=shift;
    if (defined $in and $in =~s/^\%([^{])/$1/) {
	$in='%{'.$in.'}';
    }
    return $in;
}

# drop perl deps in mga format: like
# E: Версия >='2.190.0' для 'perl-autodie' не найдена
package Mageia2ALT::DependencyFilter::PerlVer;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    return $dep if $pkgname!~/^perl[\(-]/;
    $dep->{-version}=undef;
    return $dep;
}


1;
__END__
